from jija.config.base import Base
from jija.utils.path import Path


class StructureConfig(Base):
    PROJECT_PATH = None
    CORE_PATH = None
    APPS_PATH = None
    PYTHON_PATH = None

    def __init__(self, *, project_dir=None, core_dir='core', apps_dir='apps', python_path='python'):
        StructureConfig.PROJECT_PATH = self.__get_project_path(project_dir)
        StructureConfig.CORE_PATH = StructureConfig.PROJECT_PATH + core_dir
        StructureConfig.APPS_PATH = StructureConfig.PROJECT_PATH + apps_dir
        StructureConfig.PYTHON_PATH = python_path

        super().__init__()

    @staticmethod
    def __get_project_path(project_dir):
        if isinstance(project_dir, Path):
            return project_dir

        return Path('' if project_dir is None else project_dir)



import inspect


class Base:
    INITED = False

    def __init__(self):
        if self.__class__.INITED:
            raise Exception(f'{self.__class__.__name__} already inited')

        self.__class__.INITED = True

    @classmethod
    def clean(cls):
        for key in cls.__dict__:
            if not key.startswith(f'__') and not isinstance(cls.__dict__[key], (staticmethod, classmethod)):
                setattr(cls, key, None)

        cls.INITED = False



from jija.config.base import Base


class ProjectConfig(Base):
    SECRET_KEY = None

    def __init__(self, *, secret_key=b'*' * 32):
        super().__init__()

        ProjectConfig.SECRET_KEY = secret_key



from jija.config.base import Base


class DatabaseConfig(Base):
    DATABASE = None
    PASSWORD = None
    USER = None
    PORT = None
    HOST = None

    APPS = None
    CONNECTION_LINK = None

    def __init__(self, *, database, password, host='localhost', user='postgres', port=5432):
        DatabaseConfig.DATABASE = database
        DatabaseConfig.PASSWORD = password
        DatabaseConfig.USER = user
        DatabaseConfig.PORT = port
        DatabaseConfig.HOST = host

        DatabaseConfig.connection_link = f'postgres://{user}:{password}@{host}:{port}/{database}'
        super().__init__()

    @classmethod
    def load(cls):
        from jija.apps import Apps
        cls.APPS = {}
        for app in Apps.apps.values():
            if app.database:
                cls.APPS[app.name] = {
                    "models": app.database_config,
                    "default_connection": "default",
                }

    @classmethod
    def get_config(cls):
        return {
            "connections": {
                "default": cls.CONNECTION_LINK
            },

            "apps": cls.APPS,

            'use_tz': False,
            'timezone': 'Asia/Yekaterinburg'
        }
